import React from "react";
import SwaggerClient from 'swagger-client';
import { createContext, useContext, useEffect, useState } from "react";
import { CircularProgress } from '@material-ui/core';
import { useHistory } from 'react-router';
import { useCustomSnackbar } from '../components/common/snackbar/Snackbar';

const createSwaggerClient = async () => {
  const swaggerOptions = {
    url: `${window.SERVER_ADDRESS}${window.API_BASE_ENDPOINT}`,
    requestInterceptor: (req) => {
      // if (user) req.headers["Authorization"] = `Bearer ${user.token}`;
      req.headers['Content-Type'] = 'application/json';
      req.headers['accept'] = 'application/json';
      window.req = req;
      console.log(req);
      return req;
    },
    responseInterceptor: (res) => {
      // console.log(res);
      return res;
    }
  };

  const swagger = await SwaggerClient(swaggerOptions)
  // update the server url to be our intended server
  swagger.spec.servers[0].url = `${window.SERVER_ADDRESS}`;
  window.swagger = swagger;
  return swagger;
}

export function useAPI() {
  return useAsync({ fcn: createSwaggerClient });
}

export const apiContext = createContext(null);
export const userContext = createContext({ user: null, userRoles:[], getUserRoles: ()=>{}, login: ()=>{}, logout: ()=>{} });

export function UserProvider({ children }) {
  function onLoginError(message) {
    setLoginError(message);
  }  

  const [user, setUser] = useState();
  const [loginError, setLoginError] = useState();
  const [userRoles, getUserRoles, resetUserRoles] = useUserRoles();
  const [userInfo, getUserInfo] = useUserInfo();
  const [loginResponse, loginFcn] = useLogin(onLoginError);
  const logoutFcn = useLogout();
  const history = useHistory();

  const resetLoginError = () => setLoginError(null);

  console.log("rendering UserProvider")
  console.log(user);

  useEffect(getUserInfo, [Boolean(loginResponse), getUserInfo]);
  useEffect(() => userInfo && setUser(userInfo), [setUser, userInfo]);

  useEffect(getUserRoles, [Boolean(loginResponse), getUserRoles]);

  const logout = async () => {
    console.log("Logging out!")
    setUser(null);
    resetUserRoles();
    logoutFcn();
  }

  const login = async (username, password) => {
    console.log("Logging in!");
    await loginFcn(username, password);
    await getUserRoles();
    setUser(await getUserInfo());
    console.log(history);
    console.log(userRoles);
  }

  return (
    <userContext.Provider value={{ user, userRoles, getUserRoles, login, loginError, resetLoginError, logout }}>
      {children}
    </userContext.Provider>
  );
}

export function APIProvider({ children }) {
  console.log("about to call useAPI")
  const [api] = useAPI();

  console.log(api, api?.requestInterceptor)

  return api ?
    (
      <apiContext.Provider value={api}>
        {children}
      </apiContext.Provider>
    ) :
    <CircularProgress />;
}

export function useAPIErrorHandler(onError) {
  const {logout} = useContext(userContext);
  const showError = useCustomSnackbar();

  function errorHandler(e) {
    const { response, message } = e;
    const { obj, status } = (response ?? {obj: {}, status: ""});
    if (status === 401) {
      logout();
    }
    else if (onError){
      onError(obj?.description ?? message);
    }
    else {
      showError(obj?.description ?? message);
    }
  }

  return errorHandler;
}

export function useAsync({ fcn, call = true, init = null, onError = null }) {
  const [response, setResponse] = useState(init);
  const errorHandler = useAPIErrorHandler(onError);

  const reset = () => setResponse(init);

  const f = async (...args) => {
    try {
      const r = await fcn(...args);
      console.log(r);
      setResponse(r);
    } catch (e) {
      console.warn(e);
      errorHandler(e);
    }
  };

  const [wrapper] = useState(() => f);
  useEffect(() => {
    if (call) wrapper();
  }, [wrapper, call]);
  return [response, wrapper, reset];
}

function callAndUnpack(fcn, unpacker = x => x) {
  return async (...args) => {
    console.log(...args);
    const response = await fcn(...args);
    const unpacked = unpacker(response.obj, response.status);
    return unpacked;
  };
}

export function unpackIOC(ioc) {
  console.log(ioc);
  ioc = { ...ioc };
  const { id, description, owner, createdBy, active,
     activeDeployment, hasLocalCommits, dirty, nameChangedInCCDB, nameChangedInNaming, namingName, sourceUrl, sourceVersion, sourceVersionShort } = ioc;

  let unpackedIOC = {
    id: id,
    description: description,
    owner: owner,
    createdBy: createdBy,
    activeDeployment: activeDeployment && activeDeployment.undeployment && activeDeployment.status === "SUCCESSFUL" ? null : activeDeployment,
    latestDeployment: activeDeployment,
    active: active,
    hasLocalCommits: hasLocalCommits,
    dirty: dirty,
    nameChangedInCCDB: nameChangedInCCDB,
    nameChangedInNaming: nameChangedInNaming,
    sourceUrl: sourceUrl,
    sourceVersion: sourceVersion,
    sourceVersionShort: sourceVersionShort,
    namingName: namingName
  };

  return unpackedIOC;
}

export function unpackIOCList(iocs) {
  console.log(iocs);
  return iocs.iocList.map((ioc) => unpackIOC(ioc));
}

export function useIOCList() {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.IOCs.listIocs, unpackIOCList);
  return useAsync({ fcn: method, init: [] });
}

export function useIOCSearch() {
  const api = useContext(apiContext);
  const method = callAndUnpack(
    (query) => api.apis.IOCs.listIocs({ query: query }),
    unpackIOCList
  );
  return useAsync({ fcn: method, call: false, init: [] });
}

export function useIOC(id) {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.IOCs.getIoc, unpackIOC);
  return useAsync({ fcn: method.bind(null, { iocId: id }) });
}

export function useCreateIOC(onError) {
  const api = useContext(apiContext);
  const method = callAndUnpack(
    body => api.apis.IOCs.createIoc({}, { requestBody: body }),
    unpackIOC
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function unpackDeployment(deployment) {
  const d = { ...deployment };
  console.log(d);
  return d;
}

export function unpackDeploymentList(deployments) {
  console.log(deployments);
  return deployments.deployments.map(d => unpackDeployment(d));
}

export function useDeploymentListForIOC() {
  const api = useContext(apiContext);
  const method = callAndUnpack(
    (iocId) => api.apis.Deployments.listDeployments({ iocId: iocId }),
    unpackDeploymentList
  );
  return useAsync({ fcn: method, call: false, init: [] });
}

export function unpackCommand(command) {
  const c = { ...command };
  console.log(c);
  return c;
}

export function unpackCommandList(commandList) {
  return commandList.commands.map(c => unpackCommand(c));
}

export function useCommandList(deploymentId) {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Deployments.listCommandsByDeployment, unpackCommandList)
  return useAsync({ fcn: method.bind(null, { deploymentId : deploymentId  }), init: [] });
}

export function useDeploymentSearch() {
  const api = useContext(apiContext);
  const method = callAndUnpack(
    (query) => api.apis.Deployments.listDeployments({ query: query }),
    unpackDeploymentList
  );
  return useAsync({ fcn: method, call: false, init: [] });
}

export function useDeployment(id) {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Deployments.fetchDeployment, unpackDeployment)
  return useAsync({ fcn: method.bind(null, { deploymentId: id }) });
}

export function useDeploymentById() {
  const api = useContext(apiContext);
  const method = callAndUnpack(
    (deploymentId) => api.apis.Deployments.fetchDeployment({ deploymentId: deploymentId }),
    unpackDeployment
  );
  return useAsync({ fcn: method, call: false, init: null });
}

export function useCommandById() {
  const api = useContext(apiContext);
  const method = callAndUnpack(
    (commandId) => api.apis.Deployments.fetchCommand({ commandId: commandId }),
    unpackCommand
  );
  return useAsync({ fcn: method, call: false, init: null });
}

export function unpackDeploymentJob(job) {
  return { ...job };
}

export function useDeploymentJob(awxJobId) {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Deployments.fetchDeploymentJobDetails, unpackDeploymentJob)
  return useAsync({ fcn: method.bind(null, { awxJobId: awxJobId }), call: true });
}

export function unpackCommandJob(job) {
  return { ...job };
}

export function useCommandJob(awxCommandId) {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Deployments.fetchCommandDetails, unpackCommandJob)
  return useAsync({ fcn: method.bind(null, { awxCommandId: awxCommandId }), call: true });
}

export function useUpdateAndDeployIoc(id, onError) {
  const api = useContext(apiContext);
  const method = callAndUnpack(
    body => api.apis.Deployments.updateAndDeployIoc({ iocId: id }, { requestBody: body }),
    unpackDeployment
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function useCreateUndeployment(id, onError) {
  const api = useContext(apiContext);
  const method = callAndUnpack(
    body => api.apis.Deployments.createUndeployment({iocId: id}, { requestBody: body }),
    unpackDeployment
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function unpackHost(host) {
  return { ...host };
}

export function unpackHostList(hosts) {
  return hosts.csEntryHosts.map(h => unpackHost(h));
}

export function useHostList() {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Hosts.listHosts, unpackHostList)
  return useAsync({ fcn: method, init: [] });
}

export function useHost(id) {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Hosts.findHostById, unpackHost)
  return useAsync({ fcn: method.bind(null, { hostCSEntryId: id }) });
}

export function useAvailableHost(id) {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Hosts.hostDetail, unpackHost)
  return useAsync({ fcn: method.bind(null, { hostId: id }) });
}

export function unpackCSEntryHost(host) {
  return { ...host };
}

export function unpackCSEntryHostList(hosts) {
  return hosts.csEntryHosts.map(h => unpackCSEntryHost(h));
}

export function useCSEntrySearch() {
  const api = useContext(apiContext);
  const method = callAndUnpack(
    (query) => api.apis.Hosts.listHosts({ query: query }),
    unpackCSEntryHostList
  );
  return useAsync({ fcn: method, call: false, init: [] });
}

export function unpackHostIOCList(iocs) {
  console.log({ ...iocs });
  const deployed = iocs.deployedIocs.map(ioc => unpackIOC(ioc))
  const result = { deployed };
  return result;
}

export function useHostIOCList(id) {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Hosts.findAssociatedIocsByHostId, unpackHostIOCList)
  return useAsync({ fcn: method.bind(null, { hostCSEntryId: id }), init: { deployed: [], undeployed: [] } });
}

export function unpackLogin(loginResponse) {
  return { ...loginResponse };
}

export function useLogin(onError) {
  const api = useContext(apiContext);
  const method = callAndUnpack(
    (username, password) => api.apis.Authentication.login({},{
      requestBody: {
        userName: username,
        password: password
      }
    }),
    unpackLogin
  );
  return useAsync({ fcn: method, call: false, onError: onError });
}

export function unpackUser(user) {
  return { ...user };
}

export function useUserInfo() {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Git.userInfo, unpackUser);
  return useAsync({ fcn: method, call: false });
}

export function unpackUserRoles(roles) {
  return roles;
}

export function useUserRoles() {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Authentication.getUserRoles, unpackUserRoles);
  return useAsync({ fcn: method, call: false });
}

export function useLogout() {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Authentication.logout);
  const [, logout] = useAsync({ fcn: method, call: false });
  return logout;
}

export function useStartIOC(id) {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Deployments.startIoc);
  return useAsync({ fcn: method.bind(null, {iocId: id}), call: false });
}

export function useStopIOC(id) {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Deployments.stopIoc);
  return useAsync({ fcn: method.bind(null, {iocId: id}), call: false });
}

export function useRenewToken() {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Authentication.tokenRenew);

  const [, renewToken] = useAsync({ fcn: method, call: false });
  return renewToken;
}

export function unpackLokilog(logData) {
  return { ...logData };
}

export function useLokiSysLog() {
  const api = useContext(apiContext);
  const method = callAndUnpack((hostName, timeRange) => api.apis.Loki.fetchSyslogLines({ hostName: hostName, timeRange: timeRange }), unpackLokilog)
  return useAsync({ fcn: method, call: false });
}


export function useLokiProcServLog() {
  const api = useContext(apiContext);
  const method = callAndUnpack((hostName, iocName, timeRange) => api.apis.Loki.fetchProcServLogLines({ hostName: hostName, iocName:iocName, timeRange: timeRange }), unpackLokilog)

  return useAsync({ fcn: method, call: false });
}

export function unpackStatistics(statistics) {
  return { ...statistics };
}

export function useStatistics() {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Statistics.generalStatistics, unpackStatistics)

  return useAsync({ fcn: method, call: false });
}

export function unpackNaming(naming) {
  return { ...naming };
}

export function unpackNamingList(naming) {
  return naming.map(h => unpackNaming(h));
}

export function useNamingNames() {
  const api = useContext(apiContext);
  const method = callAndUnpack((iocName) => api.apis.Naming.fetchIOCByName({iocName: iocName}), unpackNamingList)

  return useAsync({ fcn: method, call: false, init: [] });
}

export function unpackDeploymentStatistics(statistics) {
  return { ...statistics };
}

export function useDeploymentStatistics() {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Statistics.deploymentStatistics, unpackDeploymentStatistics)

  return useAsync({ fcn: method, call: false });
}

export function unpackGitReference(reference) {
  return { ...reference };
}

export function unpackTagAndCommitIdList(tagList) {
  return tagList.map(t => unpackGitReference(t));
}

export function useTagsAndCommitIds() {
  const api = useContext(apiContext);
  const method = callAndUnpack((repoUrl) => api.apis.Git.listTagsAndCommitIds({gitRepo: repoUrl}), unpackTagAndCommitIdList)
  return useAsync({ fcn: method, call: false, init: [] });
}

export function unpackIOCStatistics(statistics) {
  return { ...statistics };
}

export function useIOCStatistics() {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Statistics.iocStatistics, unpackIOCStatistics)

  return useAsync({ fcn: method, call: false });
}

export function unpackCurrentlyActiveIOCS(statistics) {
  return { ...statistics };
}

export function useCurrentlyActiveIOCs() {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Statistics.currentlyActiveIocs, unpackCurrentlyActiveIOCS)

  return useAsync({ fcn: method, call: false });
}

export function unpackActiveIOCHistory(statistics) {
  return { ...statistics };
}

export function useActiveIOCHistory() {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Statistics.activeIocHistory, unpackActiveIOCHistory)

  return useAsync({ fcn: method, call: false });
}

export function unpackDeployedIOCStat(statistics) {
  return { ...statistics };
}

export function useDeployedIOCStatistics() {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.Statistics.iocDeploymentHistory, unpackDeployedIOCStat)

  return useAsync({ fcn: method, call: false });
}

export function unpackAllowedGitProjects(projectList) {
  return projectList.toString().split(",");
}

export function useAllowedGitProjects() {
  const api = useContext(apiContext);
  const method = callAndUnpack((repoUrl) => api.apis.Git.listProjects(), unpackAllowedGitProjects)
  return useAsync({ fcn: method, call: false, init: [] });
}

export function useDeleteIOC(id, onError) {
  const api = useContext(apiContext);
  const method = callAndUnpack(api.apis.IOCs.deleteIoc, unpackDeleteIocResponse);
  return useAsync({ fcn: method.bind(null, {iocId: id}), call: false, onError: onError });
}

export function unpackDeleteIocResponse(_, status) {
  if (status === 204) {
    return "Successful delete";
  }
}

export function useUpdateIoc(id, onError) {
  const api = useContext(apiContext);
  const method = callAndUnpack(
    body => api.apis.IOCs.updateIoc({ iocId: id }, { requestBody: body }),
    unpackUpdateIoc
  );
  return useAsync({ fcn: method, call: false, onError: onError  });
}

export function unpackUpdateIoc(updateIoc) {
  const d = { ...updateIoc };
  console.log(d);
  return d;
}