import { Button, LinearProgress, Grid, Typography } from "@material-ui/core";
import { makeStyles } from '@material-ui/core/styles';
import React, { useState, useEffect, useRef, useContext } from "react";
import { useStartIOC, useStopIOC } from "../../api/SwaggerApi";
import { CommandJobStatus } from "./CommandJobStatus";
import { SimpleModal } from "../../components/common/SimpleModal/SimpleModal";
import { ConfirmationDialog } from "../dialog/ConfirmationDialog";
import { notificationContext } from "../../components/common/notification/Notifications";
import { theme } from "../../Theme";

const useStyles = makeStyles({
  startButton: {
      backgroundColor: '#4caf50',
      color: '#FFFFFF',
      "&:disabled": {
        backgroundColor: '#8d8d8d',
        color: '#FFFFFF',
      },
      "&:hover": {
        backgroundColor: '#005a00',
        color: '#FFFFFF',
      }
  },
  stopButton: {
      backgroundColor: '#aa2e25',
      color: '#FFFFFF',
      "&:disabled": {
        backgroundColor: '#8d8d8d',
        color: '#FFFFFF',
      },
      "&:hover": {
        backgroundColor: '#6c0000',
        color: '#FFFFFF',
      }
  },
});

export function IOCService({ ioc, getIOC, ongoingCommand, getCommands }) {
  const classes = useStyles(theme);

  const [startJob, startIOC, resetStartJob] = useStartIOC(ioc.id);
  const [stopJob, stopIOC, resetStopJob] = useStopIOC(ioc.id);
  const [inProgress, setInProgress] = useState(false);
  const [action, setAction] = useState(false);
  const [adHocDialogOpen, setAdHocDialogOpen] = useState(false);
  const [adHocDialogTitle, setAdHocDiatlogTitle] = useState();
  const [adHocDialogDescription, setAdHocDialogDescription] = useState();
  const [command, setCommand] = useState(null);
  const [buttonDisabled, setButtonDisabled] = useState(false);
  const callbackRef = useRef();

  const {watchCommand} = useContext(notificationContext);

  useEffect(() => {
    if (ongoingCommand) {
      if (ongoingCommand.type === "START") {
        setAction("Start");
      }
      if (ongoingCommand.type === "STOP") {
        setAction("Stop");
      }

      watchCommand(ongoingCommand.id);
      setCommand(ongoingCommand);
      setInProgress(true);
      setButtonDisabled(true);
    } else {
      setButtonDisabled(false);
    }
  }, [ongoingCommand, watchCommand]);

  useEffect(() => {
    if (startJob && (!command || command.id !== startJob.id)) {
      watchCommand(startJob.id);
      setCommand(startJob);
      setInProgress(false);
      getCommands();
    } else if (stopJob && (!command || command.id !== stopJob.id)) {
      watchCommand(stopJob.id);
      setCommand(stopJob);
      setInProgress(false);
      getCommands();
    }
  }, [startJob, stopJob, command, watchCommand, getCommands]);

  useEffect(() => {
    const interval = setInterval(() => {
      if (action && ((action === "Start" && !ioc.active) || (action === "Stop" && ioc.active))) {
        getIOC();
      }
    }, 10000);
    return () => clearInterval(interval);
  }, [action, ioc, getIOC]);

  useEffect(() => {
    const interval = setInterval(() => {
      if (ongoingCommand) {
        getCommands();
      }
    }, 2000);
    return () => clearInterval(interval);
  }, [ongoingCommand, getCommands]);

  function resetUI() {
    setCommand(null);
    setAction(null);
    resetStartJob();
    resetStopJob();
  }

  const start = async () => {
    resetUI();
    setAction("Start");
    setInProgress(true);
    setButtonDisabled(true);
    await startIOC();
  }
  const stop = async () => {
    resetUI();
    setAction("Stop");
    setInProgress(true);
    setButtonDisabled(true);
    await stopIOC();
  }

  const openStartModal = () => {
    setAdHocDiatlogTitle("Starting IOC");
    setAdHocDialogDescription(
    <>
      <Typography style={{ display: 'inline-block' }}>Are you sure want to start &nbsp;</Typography>
      <Typography style={{ fontFamily: "monospace", display: 'inline-block' }}> {ioc.name}? </Typography>
    </>);
    callbackRef.current = start;
    setAdHocDialogOpen(true);
  }

  const openStopModal = () => {
    setAdHocDiatlogTitle("Stopping IOC");
    setAdHocDialogDescription(
    <>
      <Typography style={{ display: 'inline-block' }}>Are you sure want to stop &nbsp;</Typography>
      <Typography style={{ fontFamily: "monospace", display: 'inline-block' }}> {ioc.name}? </Typography>
    </>);
    callbackRef.current = stop;
    setAdHocDialogOpen(true);
  }

  return (
    <>
      <SimpleModal open={adHocDialogOpen} setOpen={setAdHocDialogOpen}>
        <ConfirmationDialog open={adHocDialogOpen} setOpen={setAdHocDialogOpen} title={adHocDialogTitle} description={adHocDialogDescription} callback={callbackRef.current} />
      </SimpleModal>

      <Grid container spacing={1}>
        <Grid item xs={1} md={1}>
          <Button className={classes.stopButton} onClick={openStopModal} disabled={buttonDisabled}>Stop</Button>
        </Grid>
        <Grid item xs={1} md={1}>
          <Button className={classes.startButton} onClick={openStartModal} disabled={buttonDisabled}>Start</Button>
        </Grid>
        <Grid item xs={12} md={12}>
          {command ?
            <CommandJobStatus awxCommandId={command.awxCommandId} actionType={action} /> : inProgress ? <LinearProgress color="primary" /> : <></>
          }
        </Grid>
      </Grid>
    </>
  )
}
