import React from 'react';
import { CustomTable } from "../common/table/CustomTable";
import { Box, Grid, Tooltip } from "@material-ui/core";
import { useHistory } from 'react-router-dom';
import { SearchBoxFilter } from '../common/SearchBoxFilter/SearchBoxFilter';
import { IOCStatusIcon } from './IOCIcons';
import { parseHostUrl, iocHasNameDeviation } from '../common/Helper';

const ownIocsColumns = [
  { id: "status", label: 'Status', width: '8%' },
  { id: "namingName", label: 'IOC name', width: '20%', sortable: true },
  { id: "host", label: 'Host', width: '15%', sortable: true },
  { id: "network", label: 'Network', width: '15%', sortable: true },
  { id: "sourceVersion", label: 'Git reference', width: '42%', sortable: true },
]

const exploreIocsColumns = [
  { id: "status", label: 'Status', width: '8%' },
  { id: "namingName", label: 'IOC name', width: '20%', sortable: true },
  { id: "host", label: 'Host', width: '15%', sortable: true },
  { id: "network", label: 'Network', width: '15%', sortable: true },
  { id: "owner", label: 'Owner', width: '10%', sortable: true },
  { id: "sourceVersion", label: 'Git reference', width: '32%', sortable: true },
]

const hostDetailsColumns = [
  { id: "status", label: 'Status', width: '8%' },
  { id: "name", label: 'Name', width: '30%', sortable: true },
  { id: "sourceVersion", label: 'Git reference', width: '62%', sortable: true },
]

function createGitVersionField(version, shortVersion) {
  return(
    <>
      {version ?
        <Tooltip title={version} arrow enterDelay={400}>
          <label>{shortVersion}</label>
        </Tooltip>
        : "---"
      }
    </>
  )
}

function createTableRowForHostDetails(ioc) {
  const { id, name, namingName, sourceVersion, sourceVersionShort } = ioc;

  return {
    id: id,
    status: 
      <Box display="flex" justifyContent="center" p={2} m={1}>
        <IOCStatusIcon ioc={ioc} />
      </Box>,
    name: namingName ?? name,
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
  };
}

function createTableRowForOwnIocs(ioc) {
  const { id, namingName, sourceVersion, sourceVersionShort, activeDeployment, latestDeployment,
     dirty,  hasLocalCommits, serviceDisabled, essiocMissing, cellmodeUsed, nameChangedInCCDB, nameChangedInNaming } = ioc;
  const {hostName, network} =  activeDeployment ? parseHostUrl(activeDeployment.host.host) : {hostName: null, network: null};
  const hasNameDeviation = iocHasNameDeviation(ioc);
  //Marking as ERRORS
  const inconsistent =  (latestDeployment ? (latestDeployment.status !== "SUCCESSFUL") : false) || (activeDeployment && (dirty === null || hasLocalCommits === null || serviceDisabled === null || essiocMissing === null || cellmodeUsed === null));
  //Marking as WARNINGS
  //if has no ERROR, AND (not deployed, or (deployed, but has local commits,...))
  const discrepancyState = !inconsistent && ((activeDeployment && (dirty || hasLocalCommits || nameChangedInCCDB || nameChangedInNaming || hasNameDeviation)) || !activeDeployment);

  return {
    id: id,
    status: 
    <Grid container direction="column" justifyContent="center" alignItems="center">
      <IOCStatusIcon ioc={ioc} />
    </Grid>,
    namingName: namingName ?? "---",
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    host: (hostName || "---"),
    network: (network || "---"),
    discrepancy: discrepancyState,
    inconsistentState: inconsistent,
  };
}

function createTableRowForExploreIocs(ioc) {
  const { id, namingName, owner, sourceVersion, sourceVersionShort, activeDeployment, latestDeployment, dirty,  hasLocalCommits, serviceDisabled, essiocMissing, cellmodeUsed, nameChangedInCCDB, nameChangedInNaming } = ioc;
  const {hostName, network} =  activeDeployment ? parseHostUrl(activeDeployment.host.host) : {hostName: null, network: null};
  const hasNameDeviation = iocHasNameDeviation(ioc);
  //Marking as ERRORS
  const inconsistent =  (latestDeployment ? (latestDeployment.status !== "SUCCESSFUL") : false) || (activeDeployment && (dirty === null || hasLocalCommits === null || serviceDisabled === null || essiocMissing === null || cellmodeUsed === null));
  //Marking as WARNINGS
  //if has no ERROR, AND (not deployed, or (deployed, but has local commits,...))
  const discrepancyState = !inconsistent && ((activeDeployment && (dirty || hasLocalCommits || nameChangedInCCDB || nameChangedInNaming || hasNameDeviation)) || !activeDeployment);

  return {
    id: id,
    status: 
    <Grid container direction="column" justifyContent="center" alignItems="center">
      <IOCStatusIcon ioc={ioc} />
    </Grid>,
    namingName: namingName ?? "---",
    sourceVersion: createGitVersionField(sourceVersion, sourceVersionShort),
    host: (hostName || "---"),
    network: (network || "---"),
    owner: (owner || "---"),
    discrepancy: discrepancyState,
    inconsistentState: inconsistent,
  };
}

export function IOCTable({ iocs, rowType="own" }) {
  const history = useHistory();

  const onRowClicked = (id) => {
    history.push(`/iocs/${id}`);
  };

  return (
    <>
    {
      rowType === "host" ?
      <CustomTable columns={hostDetailsColumns} rows={iocs.map(ioc => createTableRowForHostDetails(ioc))} handleRowClick={onRowClicked} />
      : (rowType === "own" ?
      <CustomTable columns={ownIocsColumns} rows={iocs.map(ioc => createTableRowForOwnIocs(ioc))} handleRowClick={onRowClicked} />
      :
      <CustomTable columns={exploreIocsColumns} rows={iocs.map(ioc => createTableRowForExploreIocs(ioc))} handleRowClick={onRowClicked} />)
    }
    </>
  );
}

export function SearchableIOCTable({ iocs, setQuery, rowType }) {
  const filter = (search) => {
    setQuery(search.toLowerCase());
    return (ioc) => {
      return true;
    }
  }

  const renderFilteredIOCs = (filteredIOCs) => <IOCTable iocs={filteredIOCs} rowType={rowType}/>;

  return (
    <SearchBoxFilter items={iocs} filter={filter} render={renderFilteredIOCs} />
  );
}
