import React, { useState, useEffect } from "react";
import { makeStyles } from '@material-ui/core/styles';
import { useLokiSysLog, useLokiProcServLog } from '../../../api/SwaggerApi';
import {
    Grid,
    Select,
    MenuItem,
    InputLabel,
    FormControl,
    LinearProgress, 
    Container
} from '@material-ui/core';
import { formatDate } from '../Helper.js';
import { ConsoleOutput } from "../Console/ConsoleOutput";

const useStyles = makeStyles((theme) => ({
  formControl: {
    margin: theme.spacing(0),
    minWidth: 120,
  },
  selectEmpty: {
    marginTop: theme.spacing(2),
  },
  deployed: {
    backgroundColor:"black",
    msUserSelect : "text",
    MozUserSelect: "text",
    WebkitUserSelect: "text"
  },
  undeployed: {
    backgroundColor:"gray",
    msUserSelect : "none",
    MozUserSelect: "none",
    WebkitUserSelect: "none"
  },
}));

/**
 * host: the host URL which will be the qery parameter for backend call
 * iocName: name of ioc (optional). Used when querying procServ log
 * isSyslog: boolean, if true -> SysLog will be queried, if false -> procServLog will be queried
 * isDeployed: boolean, if true -> periodic update (backend query) will be enabled, if false -> periodic update will be disabled
 */
   
export function LokiPanel({host, iocName, isSyslog, isDeployed, shownInDialog}) {
    const classes = useStyles();
    const hostName = host.csEntryHost.name;
    const [logData, getLogData] = useLokiSysLog();
    const [procServLog, getProcServLog] = useLokiProcServLog();
    const [timeRange, setTimeRange] = React.useState(1440);
    const [periodChange, setPeriodChange] = useState(false);

    const handleChange = (event) => {
        setPeriodChange(true);
        setTimeRange(event.target.value);
    };

    //remove progressBar if intervall has been changed, and data received
    useEffect(() => {
        setPeriodChange(false);
    }, [logData, procServLog]);

    useEffect(() => {
        //query backend for procServ only if IOC is deployed; syslog is always queried
        if (isSyslog === true) { 
            getLogData(hostName, timeRange)
        } else {
            if (isDeployed === true) {
                getProcServLog(hostName, iocName, timeRange);
            }
        }
    }, [hostName, timeRange, getLogData, getProcServLog, periodChange, iocName, isDeployed, isSyslog]);
    
    useEffect(() => {
        const interval = setInterval(() => {
            //query backend periodically for procServ only if IOC is deployed; syslog is always queried
            if (isSyslog === true) { 
                getLogData(hostName, timeRange)
            } else {
                if (isDeployed === true) {
                    getProcServLog(hostName, iocName, timeRange);
                }
            }
        }, 5000);
        return () => clearInterval(interval);
    }, [hostName, timeRange, getLogData, getProcServLog, periodChange, iocName, isDeployed, isSyslog]);

    //if request period has been changed -> show progressBar
    if (periodChange) {
        return <div style={{ width: "100%" }}><LinearProgress color="primary" /></div>;
    }

    const dataReady = () => {
        return logData || procServLog;
    }

    return(
        <Container maxWidth='xl'>
        {logData || procServLog || !isDeployed ?
        <Grid container spacing={1} justify="flex-start">
            <Grid item xs={4} md={2}>
                <FormControl variant="filled" className={classes.formControl}>
                    <InputLabel id="time-range-select">Time range</InputLabel>
                        <Select
                            labelId="time-range-select"
                            value={timeRange}
                            onChange={handleChange}
                            >
                             <MenuItem value={60}>1 h</MenuItem>
                            <MenuItem value={360}>6 h</MenuItem>
                            <MenuItem value={1440}>1 day</MenuItem>
                            <MenuItem value={10080}>7 days</MenuItem>
                        </Select>
                </FormControl>
            </Grid>
            <Grid item xs={12} md={12}>
                <ConsoleOutput 
                    html={logsToPreprocess(isSyslog, logData, procServLog, isDeployed)}
                    dataReady={dataReady} 
                    extraClass={isDeployed ? classes.deployed : classes.undeployed}
                    shownInDialog={shownInDialog}
                />
            </Grid>
        </Grid>
        : <div style={{ width: "100%" }}><LinearProgress color="primary" /></div>}
        </Container>)
}

function logsToPreprocess(isSysLog, logData, procServLog, isDeployed) {
    if(isSysLog === true)  {
        return preprocessLog(logData, true)
    }  
    
    return preprocessLog(procServLog, isDeployed);
}

function formatLogLine(logLine) {
    return"<span><span class=logdate>" + formatDate(logLine.logDate) + '</span> ' + logLine.logMessage + "<br/></span>";
}

function preprocessLog(logData, isDeployed) {
    if ((logData) && (logData.lines?.length > 0)) {
        var logHtml = logData.lines.map(line => formatLogLine(line));
        return `<html>
                  <head><meta http-equiv="Content-Type" content="text/html; charset=utf-8">
                    <style type="text/css">.logdate { color: #5cb85c; } body.pre{font-family: Monaco, Menlo, Consolas, "Courier New", monospace;}
                    </style>
                  </head>
                  <body>
                    <pre>${logHtml.join('')}</pre>
                  </body>
                </html>`;
    } else { 
        if (!(isDeployed === true)) {
            return "<html><body> NOT DEPLOYED YET! </body></html>";
        }
    }

    return "<html><body> ------------ </body></html>";
}
